#define F_CPU 16000000UL // บอกไลบรารีว่า MCU ทำงานที่ 16MHz
#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

//=================
//===Define Part===
//=================

//request
#define VENDOR_RQ_GET_LIGHT0 0
#define VENDOR_RQ_GET_LIGHT1 1
#define VENDOR_RQ_GET_LIGHT2 2
#define VENDOR_RQ_GET_LIGHT3 3
#define VENDOR_RQ_GET_LIGHT4 4
#define VENDOR_RQ_GET_LIGHT5 5

//light
uint16_t A;

//===================
//===Function Part===
//===================

//read light from PCx using channel ADCx
uint16_t read_adc(uint8_t channel)
{
    ADMUX = 0b01000000;
    ADMUX |= channel;
    ADCSRA = 0b11000110;

    while ((ADCSRA & (1<<ADSC)))
        ;
    return ADCL + ADCH*256;
}


// =====================
// === USB interface ===
// =====================

usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;
    static uint8_t dataBuffer[2];  /* must stay valid when usbFunctionSetup returns */
	if (rq->bRequest == VENDOR_RQ_GET_LIGHT0)
	{
		A = read_adc(0);        
        dataBuffer[0] = A;
        dataBuffer[1] = A>>8;
        usbMsgPtr = dataBuffer;
        return 2;
	}
	else if (rq->bRequest == VENDOR_RQ_GET_LIGHT1)
	{
		A = read_adc(1);
		dataBuffer[0] = A;
		dataBuffer[1] =	A>>8;
		usbMsgPtr = dataBuffer;
		return 2;
	}
	else if (rq->bRequest == VENDOR_RQ_GET_LIGHT2)
	{
		A = read_adc(2);
		dataBuffer[0] = A;
		dataBuffer[1] =	A>>8;
		usbMsgPtr = dataBuffer;
		return 2;
	}
	else if (rq->bRequest == VENDOR_RQ_GET_LIGHT3)
	{
		A = read_adc(3);
		dataBuffer[0] = A;
		dataBuffer[1] =	A>>8;
		usbMsgPtr = dataBuffer;
		return 2;
	}
	else if (rq->bRequest == VENDOR_RQ_GET_LIGHT4)
	{
		A = read_adc(4);
		dataBuffer[0] = A;
		dataBuffer[1] =	A>>8;
		usbMsgPtr = dataBuffer;
		return 2;
	}
	else if (rq->bRequest == VENDOR_RQ_GET_LIGHT5)
	{
		A = read_adc(5);
		dataBuffer[0] = A;
		dataBuffer[1] =	A>>8;
		usbMsgPtr = dataBuffer;
		return 2;
	}
    return 0;   /* default for not implemented requests: return no data back to host */
}


//============
//=== Main ===
//============
int main()
{
	DDRC  = 0b00000000;  // PC0..PC5 = Input
    PORTC = 0b00000000;  // PC7..PC0 = 0

    usbInit();
    usbDeviceDisconnect();
    _delay_ms(300);     /* fake USB disconnect for > 250 ms */
    usbDeviceConnect();
    sei();              /* enable interrupts */
    while (1)           /* main event loop */
    {                
        usbPoll();      /* คำสั่งนี้ต้องถูกเรียกอย่างน้อยที่สุดทุก ๆ 50ms */
    }
    return 0;
}
